//
//  ListingViewController.swift
//  TimeTecBLEDemo
//
//  Created by Nicholas Chong on 21/08/2019.
//  Copyright © 2019 TimeTec Cloud Sdn Bhd. All rights reserved.
//  Swift 5.0 Demo
//

import UIKit
import CoreBluetooth
import TimeTecBLE

/*
 Kindly refer to these documentation for more info:
 https://developer.apple.com/documentation/corebluetooth
 https://www.raywenderlich.com/231-core-bluetooth-tutorial-for-ios-heart-rate-monitor
 
 Simple walkthrough on Bluetooth Low Energy (BLE)
 1) ensure Bluetooth state is ON
 2) start scanning for nearby peripherals
 3) connect peripheral name (MAC Address for TimeTec BLE products)
 4) stop scanning for other peripherals
 5) discover service characteristic for that particular peripheral
 6) write value including encryption to unlock TimeTec BLE products
 */

class ListingViewController: UIViewController {
    
    @IBOutlet weak var segmentedController: UISegmentedControl!
    @IBOutlet weak var collectionView: UICollectionView!
    
    var centralManager: CBCentralManager!
    var instancePeripheral: CBPeripheral!
    var instanceCharacteristic: CBCharacteristic!
    
    var bleData: BLE!
    
    let alertConnecting = UIAlertController(title: "Searching TimeTec BLE", message: "Searching for device...", preferredStyle: UIAlertController.Style.alert)
    var connectionTimer: Timer?
    var bluetoothStateOff: Bool = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
        initialize()
    }
    
    func initialize() {
        switch bleData.bleType {
        case .BLE2:
            segmentedController.selectedSegmentIndex = 0
            segmentedController.setEnabled(false, forSegmentAt: 1)
        case .BLE16:
            segmentedController.selectedSegmentIndex = 1
            segmentedController.setEnabled(false, forSegmentAt: 0)
        }
        
    }
    
    func writeValue() {
        if let indexPath = self.collectionView.indexPathsForSelectedItems?.first {
            
            print("writing to characteristic")
            // ensure your indexPath is same as the channel input
            // TODO: - BLE Data Encryption
            instancePeripheral.writeValue(TimeTecBLE.shared.getBLEData(bleType: (bleData.bleType == .BLE2) ? .BLE2 : .BLE16, channel: (indexPath.row + 1), macAddress: bleData.macAddress), for: instanceCharacteristic, type: .withResponse)
            
        }
    }
    
    // MARK: - Event Actions
    @objc func showBluetoothAlert(_ sender: AnyObject) {
        
        print("bluetooth not powered on")
        alertConnecting.dismiss(animated: false, completion: nil)
        
        let alert = UIAlertController(title: "Failure", message: "Bluetooth is not turn on.", preferredStyle: UIAlertController.Style.alert)
        alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: {(_: UIAlertAction!) in
        }))
        self.present(alert, animated: true, completion: nil)
        
    }
    
    @objc func showConnectingAlert(_ sender: AnyObject) {
        print("connecting to peripheral...")
        self.present(alertConnecting, animated: true, completion: nil)
    }
    
    @objc func showFailureAlert(_ sender: AnyObject) {
        
        centralManager.stopScan()
        print("central disconnected")
        alertConnecting.dismiss(animated: false, completion: nil)
        
        if !bluetoothStateOff {
            let alert = UIAlertController(title: "Failure", message: "Device not found", preferredStyle: UIAlertController.Style.alert)
            
            alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: {(_: UIAlertAction!) in
            }))
            self.present(alert, animated: true, completion: nil)
            
            DispatchQueue.main.asyncAfter(deadline: .now() + 5) {
                alert.dismiss(animated: false, completion: nil)
            }
        }
        
    }
    
}

// MARK: - Extension UICollectionViewDataSource, UICollectionViewDelegate, UICollectionViewDelegateFlowLayout
extension ListingViewController: UICollectionViewDataSource, UICollectionViewDelegate, UICollectionViewDelegateFlowLayout {
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        switch bleData.bleType {
        case .BLE2:
            // by default using channel 1 only, return 1
            return 3
        case .BLE16:
            return 16
        }
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "collectioncell", for: indexPath) as! CollectionViewCell
        cell.labelChannel.text = String(indexPath.row + 1)
        
        return cell
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        
        let width = ((view.bounds.size.width - 40) / 3)
        return CGSize(width: width, height: width)
        
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        
        self.centralManager = CBCentralManager(delegate: self, queue: nil)
        self.bluetoothStateOff = false
        
    }
    
}

// MARK: - Extension CBCentralManagerDelegate
extension ListingViewController: CBCentralManagerDelegate {
    func centralManagerDidUpdateState(_ central: CBCentralManager) {
        
        switch central.state {
        case .unknown:
            print("central.state is .unknown")
            centralManager.stopScan()
        case .resetting:
            print("central.state is .resetting")
            centralManager.stopScan()
        case .unsupported:
            print("central.state is .unsupported")
            centralManager.stopScan()
        case .unauthorized:
            print("central.state is .unauthorized")
            centralManager.stopScan()
        case .poweredOff:
            print("central.state is .poweredOff")
            centralManager.stopScan()
            showBluetoothAlert(self)
            bluetoothStateOff = true
        case .poweredOn:
            print("central.state is .poweredOn")
            if !bluetoothStateOff {
                centralManager.scanForPeripherals(withServices: nil)
                connectionTimer = Timer.scheduledTimer(timeInterval: 7.0, target: self, selector: #selector(showFailureAlert(_:)), userInfo: nil, repeats: false)
                showConnectingAlert(self)
            }
        @unknown default:
            print("unknown")
        }
        
    }
    
    func centralManager(_ central: CBCentralManager, didDiscover peripheral: CBPeripheral, advertisementData: [String : Any], rssi RSSI: NSNumber) {
        
        // as Apple INC hides the MAC Address and only generates UUID, different devices will get different UUID for the same peripheral. In order to simplify developers, MAC Address can be found on TimeTec products peripheral name.
        if peripheral.name != nil {
            if peripheral.name! == bleData.macAddress {
                
                instancePeripheral = peripheral
                instancePeripheral.delegate = self
                central.stopScan()
                central.connect(instancePeripheral, options: nil)
                
            }
        }
        
    }
    
    func centralManager(_ central: CBCentralManager, didConnect peripheral: CBPeripheral) {
        print("peripheral connected")
        instancePeripheral.discoverServices(nil)
    }
    
    func centralManager(_ central: CBCentralManager, didDisconnectPeripheral peripheral: CBPeripheral, error: Error?) {
        print("peripheral disconnected")
    }
    
}

// MARK: - Extension CBPeripheralDelegate
extension ListingViewController: CBPeripheralDelegate {
    func peripheral(_ peripheral: CBPeripheral, didDiscoverServices error: Error?) {
        
        guard let services = peripheral.services else { return }
        
        for service in services {
            peripheral.discoverCharacteristics(nil, for: service)
        }
        
    }
    
    func peripheral(_ peripheral: CBPeripheral, didDiscoverCharacteristicsFor service: CBService, error: Error?) {
        service.characteristics?.forEach({ (characteristic) in
            
            // this is important as TimeTec BLE products using this specific Characteristic UUID
            if characteristic.uuid.uuidString == TimeTecBLE.shared.peripheralCharacteristicUUID {
                print("characteristic connected")
                instanceCharacteristic = characteristic
                writeValue()
            }
            
        })
        
    }
    
    func peripheral(_ peripheral: CBPeripheral, didWriteValueFor characteristic: CBCharacteristic, error: Error?) {
        print("succesfully wrote to characteristic")
       
        connectionTimer?.invalidate()
        alertConnecting.dismiss(animated: false, completion: nil)
        centralManager.cancelPeripheralConnection(peripheral)
        // additional setup to change the button image (to green light) and add a timer for 5 seconds to change back into original state
        
    }
    
}
